#pragma once

#include "ActorInterface_Private.h"
#include "NpMsgr.h"
#include "Mutex.h"
#include "UtilFunction.h"

class CBuddyUser : public IGTBuddyUser
{
public:
	CBuddyUser(IBuddy *pBuddy) : m_pBuddy(pBuddy)
	{
		//m_isFakeBuddy = false;
        m_mutex.SetRecursive();
	}
	virtual ~CBuddyUser(void);

	// For GTObject
protected:	
	virtual void OnReleaseObject(void) { delete this; }

public:	
	virtual int GetID(void) const
	{
        GT_ASSERT(m_sData.GetFlags() & GT_BUDDY_DATA_FLAG_ID);
		return m_sData.GetID();
	}

	virtual int GetGroupID(void) const
	{
        GT_ASSERT(m_sData.GetFlags() & GT_BUDDY_DATA_FLAG_GROUPID);
		return m_sData.GetGroupID();
	}
	
	virtual BYTE GetType(void) const;
	virtual bool IsOnline(void) const;
	virtual bool IsBuddy(void) const;
	//virtual bool IsFakeBuddy(void) const;
	virtual bool IsBlock(void) const;
	virtual bool IsWait(void) const;

	virtual bool IsMobileUser(void) const
	{
        if (m_sData.GetFlags() & GT_BUDDY_DATA_FLAG_ESTATUS)
        {
            GTUserStatusType status = GetStatus();
            if(status != ustOFFLINE)
            {
                COMMON_OS_TYPE _osType = ConvertUtil::ConvertToInternalOSType(GetOSType());
                IF_COMMON_CHECK_MOBILE_OS_TYPE(_osType)
                {
                    return true;
                }
                return false;
            }
        }
    	return m_sData.GetMobileUsage();			
	}
	
	virtual GTOSType GetOSType(void) const;
	
	virtual LPCWSTR GetUserID(void) const
	{
		return m_sData.GetUserID().c_str();
	}
	virtual LPCWSTR GetTalkName(void) const
	{
		return m_sData.GetTalkName().c_str();
	}
	virtual LPCWSTR GetAlias(void) const
	{
		return m_sData.GetAlias().c_str();
	}
	virtual LPCWSTR GetName(void) const
	{
		return m_sData.GetName().c_str();
	}

	virtual LPCWSTR GetDisplayName(void) const;

    virtual DWORD GetProfileImageCRC(void) const
    {
        GT_ASSERT(m_sData.GetFlags() & GT_BUDDY_DATA_FLAG_IMAGECRC);
        return m_sData.GetProfileImageCRC();
    }
    
	virtual GTUserStatusType GetStatus(void) const
	{
        GT_ASSERT(m_sData.GetFlags() & GT_BUDDY_DATA_FLAG_ESTATUS);
		return m_sData.GetStatus();
	}

	virtual NTIPTStatusType GetIPTStatus(void) const
	{
        GT_ASSERT(m_sData.GetFlags() & GT_BUDDY_DATA_FLAG_IPTSTATUS);
		return m_sData.GetIPTStatus();
	}

	virtual const char* GetCustomData(void) const;
	virtual int GetCustomDataSize(void) const;

	virtual void GetUpdatableData(GT_BUDDY_UPDATABLE_DATA &sData) const
	{
		m_sData.GetUpdatableData(sData);
	}

	inline IBuddy * GetBuddyPtr(void)
	{
		return m_pBuddy;
	}

	void SetBuddyPtr(IBuddy *pBuddy)
	{
		m_pBuddy = pBuddy;
	}

	inline void SetGroupID(int nGroupID)
	{
        m_sData.SetGroupID(nGroupID);
	}

	void SetTalkName(LPCWSTR strTalkName)
	{
		m_strTalkName = strTalkName;
	}

	void SetData(const GT_BUDDY_DATA &userdata)
	{
		m_sData.Load(userdata);
	}
	void SetData(const P_GT_BUDDY_DATA &userdata)
	{
		m_sData = userdata;
	}

	void LoadData(const GT_BUDDY_UPDATABLE_DATA &userdata);
	void LoadData(const P_GT_BUDDY_UPDATABLE_DATA &userdata);
	
	inline const P_GT_BUDDY_DATA & GetData()
	{
		return m_sData;
	}
	inline void GetData(GT_BUDDY_DATA &sData)
	{
		m_sData.Convert(sData);
	}

	void CollectData();
	
	void ReleaseUser()
	{
		m_mutex.Lock();
		m_pBuddy = NULL;
        m_mutex.Unlock();
    
//        GT_PLOG("CBuddyUser >> ReleaseUser : %p\n", this);
		ReleaseObject();
	}

	static bool CollectBuddyInfo(IBuddy *pBuddy, GT_BUDDY_DATA &sInfo);
	static bool CollectBuddyInfo(IBuddy *pBuddy, P_GT_BUDDY_DATA &sInfo);

private:
	P_GT_BUDDY_DATA m_sData;
	IBuddy *m_pBuddy;
	mutable nfc::CMutex m_mutex;

	WSTRING m_strTalkName;
	//bool	m_isFakeBuddy;
};

typedef std::vector<CBuddyUser *> buddyvector_t;
typedef std::map<WSTRING, CBuddyUser *> buddymap_t;
typedef std::map<WSTRING, NE_MEMBER_DATA> employeemap_t;
